"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initAppInsights = exports.disableSamplingByTag = exports.removeQueryParamsPreprocessor = void 0;
const appInsights = require("applicationinsights");
const applicationinsights_1 = require("applicationinsights");
const agent_1 = require("./agent");
/**
 * Internal usage, do not export
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function startAppInsights(connectionString, aiConfig) {
    var _a, _b, _c, _d;
    const ai = appInsights.setup(connectionString);
    if (aiConfig.isTracingDisabled) {
        ai.setAutoCollectConsole(false)
            .setAutoCollectPerformance(false)
            .setAutoCollectDependencies(false)
            .setAutoCollectRequests(false)
            .setAutoDependencyCorrelation(false);
    }
    // @see https://github.com/Azure/azure-functions-host/issues/3747
    // @see https://github.com/Azure/azure-functions-nodejs-worker/pull/244
    ai.setDistributedTracingMode(applicationinsights_1.DistributedTracingModes.AI_AND_W3C)
        .setSendLiveMetrics(true)
        // @see https://stackoverflow.com/questions/49438235/application-insights-metric-in-aws-lambda/49441135#49441135
        .setUseDiskRetryCaching(false)
        .start();
    appInsights.defaultClient.addTelemetryProcessor(
    // eslint-disable-next-line @typescript-eslint/no-use-before-define
    removeQueryParamsPreprocessor);
    // eslint-disable-next-line @typescript-eslint/no-use-before-define
    appInsights.defaultClient.addTelemetryProcessor(disableSamplingByTag);
    // Configure the data context of the telemetry client
    // refering to the current application version with a specific CloudRole
    if (aiConfig.applicationVersion !== undefined) {
        // eslint-disable-next-line functional/immutable-data
        appInsights.defaultClient.context.tags[appInsights.defaultClient.context.keys.applicationVersion] = aiConfig.applicationVersion;
    }
    if (aiConfig.cloudRole !== undefined) {
        // eslint-disable-next-line functional/immutable-data
        appInsights.defaultClient.context.tags[appInsights.defaultClient.context.keys.cloudRole] = aiConfig.cloudRole;
    }
    // override some default values when provided
    const config = appInsights.defaultClient.config;
    // eslint-disable-next-line functional/immutable-data
    config.httpAgent = (_a = aiConfig.httpAgent) !== null && _a !== void 0 ? _a : config.httpAgent;
    // eslint-disable-next-line functional/immutable-data
    config.httpsAgent = (_b = aiConfig.httpsAgent) !== null && _b !== void 0 ? _b : config.httpsAgent;
    // eslint-disable-next-line functional/immutable-data
    config.samplingPercentage =
        (_c = aiConfig.samplingPercentage) !== null && _c !== void 0 ? _c : config.samplingPercentage;
    // eslint-disable-next-line functional/immutable-data
    config.disableAppInsights =
        (_d = aiConfig.disableAppInsights) !== null && _d !== void 0 ? _d : config.disableAppInsights;
    return appInsights.defaultClient;
}
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function removeQueryParamsPreprocessor(envelope, _) {
    if (envelope.data.baseType === "RequestData") {
        const originalUrl = envelope.data.baseData.url;
        // eslint-disable-next-line functional/immutable-data
        envelope.data.baseData.url =
            originalUrl.split("?")[0];
    }
    return true;
}
exports.removeQueryParamsPreprocessor = removeQueryParamsPreprocessor;
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function disableSamplingByTag(envelope, _) {
    if (envelope.tags.samplingEnabled === "false") {
        // eslint-disable-next-line functional/immutable-data
        envelope.sampleRate = 100;
    }
    return true;
}
exports.disableSamplingByTag = disableSamplingByTag;
/**
 * Configure Application Insights default client
 * using settings taken from the environment:
 *
 * - setup tracing options
 * - setup cloudRole and version
 * - eventually setup http keeplive to prevent SNAT port exhaustion
 * - start application insights
 *
 * As the default client is a singleton shared between functions
 * you may want to prevent bootstrapping insights more than once
 * checking if appInsights.defaultClient id already set in the caller.
 *
 * To enable http agent keepalive set up these environment variables:
 * https://github.com/pagopa/io-ts-commons/blob/master/src/agent.ts#L11
 *
 * If you need to programmatically call Application Insights methods
 * set operationId = context.Tracecontext.traceparent to correlate
 * the call with the parent request.
 *
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function initAppInsights(aiConnectionString, config, env = process.env) {
    // @see https://github.com/pagopa/io-ts-commons/blob/master/src/agent.ts
    // @see https://docs.microsoft.com/it-it/azure/load-balancer/load-balancer-outbound-connections
    const agentOpts = (0, agent_1.isFetchKeepaliveEnabled)(env)
        ? {
            httpAgent: (0, agent_1.newHttpAgent)((0, agent_1.getKeepAliveAgentOptions)(env)),
            httpsAgent: (0, agent_1.newHttpsAgent)((0, agent_1.getKeepAliveAgentOptions)(env)),
        }
        : {};
    // defaults to the name of the function app if not set in config
    const cloudRole = (config === null || config === void 0 ? void 0 : config.cloudRole) || env.WEBSITE_SITE_NAME;
    return startAppInsights(aiConnectionString, Object.assign(Object.assign({ cloudRole }, config), agentOpts));
}
exports.initAppInsights = initAppInsights;
//# sourceMappingURL=appinsights.js.map