"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommaSeparatedListOf = void 0;
const t = require("io-ts");
/**
 * Create a decoder that parses a string containing a list of comma-separated elements into an array of typed items, using the provided decoder
 *
 * @param decoder a io-ts decoder
 *
 * @returns either a decode error or the array of decoded items
 */
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
const CommaSeparatedListOf = (decoder) => new t.Type(`CommaSeparatedListOf<${decoder.name}>`, (value) => Array.isArray(value) && value.every((e) => decoder.is(e)), (input) => t.readonlyArray(decoder).decode(typeof input === "string"
    ? input
        .split(",")
        .map((e) => e.trim())
        .filter(Boolean)
    : input // it should not happen, but in case we let the decoder fail
), String);
exports.CommaSeparatedListOf = CommaSeparatedListOf;
//# sourceMappingURL=comma-separated-list.js.map