import * as t from "io-ts";
/**
 * Accepts short formats (ie. "2018-10-13")
 * with or without time and timezone parts.
 */
export declare const DateFromString: t.Type<Date, string, unknown>;
export type DateFromString = t.TypeOf<typeof DateFromString>;
export declare const patternDateFromString: (pattern: string, tagName: string) => t.Type<Date, string>;
/**
 * Accepts only full ISO8601 format with UTC (Z) timezone
 *
 * ie. "2018-10-13T00:00:00.000Z"
 */
export declare const UtcOnlyIsoDateFromString: t.Type<Date, string, unknown>;
export type UtcOnlyIsoDateFromString = t.TypeOf<typeof UtcOnlyIsoDateFromString>;
/**
 * Accepts only full ISO8601 format with +/-00:00 timezone
 *
 * ie. "2018-10-13T00:00:00.000+00:00"
 */
export declare const TimezoneOnlyIsoDateFromString: t.Type<Date, string, unknown>;
export type TimezoneOnlyIsoDateFromString = t.TypeOf<typeof TimezoneOnlyIsoDateFromString>;
export declare const IsoDateFromString: t.UnionC<[t.Type<Date, string, unknown>, t.Type<Date, string, unknown>]>;
export type IsoDateFromString = t.TypeOf<typeof IsoDateFromString>;
/**
 * @deprecated use IsoDateFromString instead.
 */
export declare const UTCISODateFromString: t.UnionC<[t.Type<Date, string, unknown>, t.Type<Date, string, unknown>]>;
/**
 * @deprecated use IsoDateFromString instead.
 */
export type UTCISODateFromString = t.TypeOf<typeof UTCISODateFromString>;
/**
 * Accepts only a valid timestamp format (number)
 *
 * ie. 1577836800000
 */
export declare const DateFromTimestamp: t.Type<Date, number, unknown>;
export type DateFromTimestamp = t.TypeOf<typeof DateFromTimestamp>;
