"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toPlainText = exports.toEncryptedPayload = exports.EncryptedPayload = void 0;
const crypto = require("crypto");
const E = require("fp-ts/lib/Either");
const t = require("io-ts");
// AES-128-CBC requires 16 random bytes for pseudo randomic string used in crypto algorithm
const IV_LENGTH = 16;
// aes-128-cbc uses a 128 bit key (16 bytes * 8)
const AES_KEY_LENGTH = 16;
exports.EncryptedPayload = t.interface({
    // encrypted text (Base64)
    cypherText: t.string,
    // random string to feed AES (Base64)
    iv: t.string,
    // AES Key encrypted with RSA public key (Base64)
    // eslint-disable-next-line sort-keys
    encryptedKey: t.string,
});
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function toEncryptedPayload(rsaPubKey, plainText) {
    return E.tryCatch(() => {
        const iv = crypto.randomBytes(IV_LENGTH);
        const aesKey = crypto.randomBytes(AES_KEY_LENGTH);
        const cipher = crypto.createCipheriv("aes-128-cbc", aesKey, iv);
        // @see https://nodejs.org/api/crypto.html#crypto_class_cipher
        const cypherText = Buffer.concat([
            cipher.update(plainText),
            cipher.final(),
        ]).toString("base64");
        const encryptedKey = crypto
            .publicEncrypt(rsaPubKey, aesKey)
            .toString("base64");
        return {
            cypherText,
            encryptedKey,
            iv: iv.toString("base64"),
        };
    }, E.toError);
}
exports.toEncryptedPayload = toEncryptedPayload;
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function toPlainText(rsaPrivateKey, encryptedPayload) {
    return E.tryCatch(() => {
        const iv = Buffer.from(encryptedPayload.iv, "base64");
        const aesKey = crypto.privateDecrypt(rsaPrivateKey, Buffer.from(encryptedPayload.encryptedKey, "base64"));
        const decipher = crypto.createDecipheriv("aes-128-cbc", Buffer.from(aesKey), iv);
        const decrypted = decipher.update(Buffer.from(encryptedPayload.cypherText, "base64"));
        const plainText = Buffer.concat([decrypted, decipher.final()]);
        return plainText.toString("utf-8");
    }, E.toError);
}
exports.toPlainText = toPlainText;
//# sourceMappingURL=encrypt.js.map