/// <reference types="node" />
import * as t from "io-ts";
export declare enum NodeEnvironmentEnum {
    DEVELOPMENT = "dev",
    PRODUCTION = "production"
}
/**
 * A type for describing the node runtime environment
 */
export declare const NodeEnvironment: t.Type<NodeEnvironmentEnum, NodeEnvironmentEnum, unknown>;
export type NodeEnvironment = t.TypeOf<typeof NodeEnvironment>;
/**
 * Resolves the Node environment from the process environment.
 * Defaults to PRODUCTION if no environment is set.
 *
 * @param env The environment variables (defaults to `process.env`)
 * @param defaultEnv The default environment (`NodeEnvironmentEnum.PRODUCTION`)
 */
export declare function getNodeEnvironmentFromProcessEnv(env?: NodeJS.ProcessEnv, defaultEnv?: NodeEnvironment): NodeEnvironment;
