"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNodeEnvironmentFromProcessEnv = exports.NodeEnvironment = exports.NodeEnvironmentEnum = void 0;
const types_1 = require("./types");
var NodeEnvironmentEnum;
(function (NodeEnvironmentEnum) {
    NodeEnvironmentEnum["DEVELOPMENT"] = "dev";
    NodeEnvironmentEnum["PRODUCTION"] = "production";
})(NodeEnvironmentEnum = exports.NodeEnvironmentEnum || (exports.NodeEnvironmentEnum = {}));
/**
 * A type for describing the node runtime environment
 */
exports.NodeEnvironment = (0, types_1.enumType)(NodeEnvironmentEnum, "NodeEnvironment");
/**
 * Resolves the Node environment from the process environment.
 * Defaults to PRODUCTION if no environment is set.
 *
 * @param env The environment variables (defaults to `process.env`)
 * @param defaultEnv The default environment (`NodeEnvironmentEnum.PRODUCTION`)
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function getNodeEnvironmentFromProcessEnv(env = process.env, defaultEnv = NodeEnvironmentEnum.PRODUCTION) {
    const nodeEnv = env.NODE_ENV;
    if (nodeEnv === NodeEnvironmentEnum.DEVELOPMENT) {
        return NodeEnvironmentEnum.DEVELOPMENT;
    }
    else if (nodeEnv === NodeEnvironmentEnum.PRODUCTION) {
        return NodeEnvironmentEnum.PRODUCTION;
    }
    return defaultEnv;
}
exports.getNodeEnvironmentFromProcessEnv = getNodeEnvironmentFromProcessEnv;
//# sourceMappingURL=environment.js.map