"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getIsUserEligibleForNewFeature = exports.FeatureFlag = exports.FeatureFlagEnum = void 0;
const types_1 = require("./types");
var FeatureFlagEnum;
(function (FeatureFlagEnum) {
    FeatureFlagEnum["ALL"] = "ALL";
    FeatureFlagEnum["BETA"] = "BETA";
    FeatureFlagEnum["CANARY"] = "CANARY";
    FeatureFlagEnum["NONE"] = "NONE";
})(FeatureFlagEnum = exports.FeatureFlagEnum || (exports.FeatureFlagEnum = {}));
exports.FeatureFlag = (0, types_1.enumType)(FeatureFlagEnum, "FeatureFlag");
/**
 * Return a function that takes an input of type T and returns whether it's eligible for the feature,
 * based on the feature flag level:
 *
 * "NONE"   -> no one is enabled
 * "BETA"   -> only user that pass the `isUserBeta` check are enabled
 * "CANARY" -> only user that pass the `isUserBeta` or the `isUserCanary` check are enabled
 * "ALL"    ->  every user is enabled
 *
 * @param isUserBeta The function that checks if the user is enabled for beta testing
 * @param isUserCanary The function that checks if the user is enabled for canary testing
 * @param featureFlag The Feature Flag level defined
 */
const getIsUserEligibleForNewFeature = (isUserBeta, isUserCanary, featureFlag) => (i) => {
    switch (featureFlag) {
        case "ALL":
            return true;
        case "BETA":
            return isUserBeta(i);
        case "CANARY":
            return isUserCanary(i) || isUserBeta(i);
        case "NONE":
            return false;
        default:
            return false;
    }
};
exports.getIsUserEligibleForNewFeature = getIsUserEligibleForNewFeature;
//# sourceMappingURL=featureFlag.js.map