"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemverFromFromUserAgentString = exports.UserAgentSemverValid = exports.UserAgentSemver = void 0;
const function_1 = require("fp-ts/lib/function");
const E = require("fp-ts/lib/Either");
const t = require("io-ts");
const semver = require("semver");
const strings_1 = require("./strings");
const reporters_1 = require("./reporters");
exports.UserAgentSemver = t.type({
    clientName: strings_1.NonEmptyString,
    clientVersion: strings_1.Semver,
});
/**
 * this comparator check if the second provided userAgentSemver's version is
 * prior or equal to the first one
 */
exports.UserAgentSemverValid = {
    equals: (first, second) => first.clientName === second.clientName &&
        semver.satisfies(second.clientVersion, `<=${first.clientVersion}`),
};
/**
 * This custom decoder validates a string format (i.e AppName/<version>)
 * into a valid UserAgentSemver type object.
 * Encoder turns a UserAgentSemver object into a string representation.
 */
exports.SemverFromFromUserAgentString = new t.Type("SemverFromFromUserAgent", (u) => (0, function_1.pipe)(u, t.string.decode, E.fold(() => false, (s) => (0, function_1.pipe)(s.substring(s.indexOf(`/`) + 1), (ver) => ({
    clientName: s.substring(0, s.indexOf(`/`)),
    clientVersion: ver
        // ignore extra user agents information
        .split(" ")[0]
        .split(".")
        .slice(0, 3)
        .join("."),
}), exports.UserAgentSemver.is))), (s, ctx) => (0, function_1.pipe)(s, t.string.decode, E.mapLeft((errs) => Error((0, reporters_1.errorsToReadableMessages)(errs).join("|"))), E.map((str) => (0, function_1.pipe)(str.substring(str.indexOf(`/`) + 1), (ver) => ({
    clientName: str.substring(0, str.indexOf(`/`)),
    clientVersion: ver
        // ignore extra user agents information
        .split(" ")[0]
        .split(".")
        .slice(0, 3)
        .join("."),
}))), E.chainW((0, function_1.flow)(exports.UserAgentSemver.decode, E.mapLeft((errs) => Error((0, reporters_1.errorsToReadableMessages)(errs).join("|"))))), E.fold((e) => t.failure(s, ctx, e.message), t.success)), (u) => `${u.clientName}/${u.clientVersion}`);
//# sourceMappingURL=http-user-agent.js.map