import * as t from "io-ts";
import * as E from "fp-ts/lib/Either";
import * as J from "fp-ts/lib/Json";
/**
 * This is the JWK JSON type for the EC keys.
 */
export declare const ECKey: t.ExactC<t.TypeC<{
    crv: t.StringC;
    kty: t.LiteralC<"EC">;
    x: t.StringC;
    y: t.StringC;
}>>;
export type ECKey = t.TypeOf<typeof ECKey>;
/**
 * This is the JWK JSON type for the RSA keys.
 */
export declare const RSAKey: t.ExactC<t.TypeC<{
    alg: t.StringC;
    e: t.StringC;
    kty: t.LiteralC<"RSA">;
    n: t.StringC;
}>>;
export type RSAKey = t.TypeOf<typeof RSAKey>;
/**
 * The Public Key JWK type. It could be either an ECKey or an RSAKey.
 */
export declare const JwkPublicKey: t.UnionC<[t.ExactC<t.TypeC<{
    alg: t.StringC;
    e: t.StringC;
    kty: t.LiteralC<"RSA">;
    n: t.StringC;
}>>, t.ExactC<t.TypeC<{
    crv: t.StringC;
    kty: t.LiteralC<"EC">;
    x: t.StringC;
    y: t.StringC;
}>>]>;
export type JwkPublicKey = t.TypeOf<typeof JwkPublicKey>;
export declare const parseJwkOrError: (token: unknown) => E.Either<Error, J.Json>;
export declare const JwkPublicKeyFromToken: t.Type<{
    crv: string;
    kty: "EC";
    x: string;
    y: string;
} | {
    alg: string;
    e: string;
    kty: "RSA";
    n: string;
}, string, unknown>;
export type JwkPublicKeyFromToken = t.TypeOf<typeof JwkPublicKeyFromToken>;
