"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JwkPublicKeyFromToken = exports.parseJwkOrError = exports.JwkPublicKey = exports.RSAKey = exports.ECKey = void 0;
const t = require("io-ts");
const E = require("fp-ts/lib/Either");
const J = require("fp-ts/lib/Json");
const function_1 = require("fp-ts/lib/function");
const jose = require("jose");
const strings_1 = require("./strings");
const reporters_1 = require("./reporters");
/**
 * This is the JWK JSON type for the EC keys.
 */
exports.ECKey = t.exact(t.type({
    crv: t.string,
    kty: t.literal("EC"),
    x: t.string,
    y: t.string,
}));
/**
 * This is the JWK JSON type for the RSA keys.
 */
exports.RSAKey = t.exact(t.type({
    alg: t.string,
    e: t.string,
    kty: t.literal("RSA"),
    n: t.string,
}));
/**
 * The Public Key JWK type. It could be either an ECKey or an RSAKey.
 */
exports.JwkPublicKey = t.union([exports.RSAKey, exports.ECKey], "JwkPublicKey");
const parseJwkOrError = (token) => (0, function_1.pipe)(token, strings_1.NonEmptyString.decode, E.mapLeft(E.toError), E.chain((tokenStr) => E.tryCatch(() => (0, function_1.pipe)(Buffer.from(jose.base64url.decode(tokenStr)), E.fromPredicate((b) => b.length > 0, () => {
    throw new Error("Unexpected JWK empty buffer");
}), E.map((b) => b.toString()), E.toUnion), (_) => Error("Cannot decode JWK Base64"))), E.chain((0, function_1.flow)(J.parse, E.mapLeft((_) => Error("Cannot parse JWK to JSON format")))), E.orElseW((error) => (0, function_1.pipe)(token, E.fromPredicate(exports.JwkPublicKey.is, () => error))));
exports.parseJwkOrError = parseJwkOrError;
exports.JwkPublicKeyFromToken = new t.Type("JwkPublicKeyFromToken", (s) => (0, function_1.pipe)(s, exports.parseJwkOrError, E.toUnion, exports.JwkPublicKey.is), (s, ctx) => (0, function_1.pipe)(s, exports.parseJwkOrError, E.chainW((0, function_1.flow)(exports.JwkPublicKey.decode, E.mapLeft((errs) => Error((0, reporters_1.errorsToReadableMessages)(errs).join("|"))))), E.fold((e) => t.failure(s, ctx, e.message), t.success)), (0, function_1.flow)(J.stringify, E.map(jose.base64url.encode), E.getOrElseW((_) => {
    throw new Error("Cannot stringify a malformed json");
})));
//# sourceMappingURL=jwk.js.map