import * as jwt from "jsonwebtoken";
import * as TE from "fp-ts/TaskEither";
import { Second } from "./units";
import { NonEmptyString } from "./strings";
/**
 * JWT Generation
 */
export type GetGenerateJWT = <T extends Record<string, unknown>>(issuer: NonEmptyString, primaryPrivateKey: NonEmptyString) => (payload: T, ttl: Second) => TE.TaskEither<Error, NonEmptyString>;
export declare const getGenerateJWT: GetGenerateJWT;
export declare const errorIsInvalidSignatureError: (error: Error) => boolean;
/**
 * JWT Validation
 */
export type GetValidateJWT = (issuer: NonEmptyString, primaryPublicKey: NonEmptyString, secondaryPublicKey?: NonEmptyString) => (token: NonEmptyString) => TE.TaskEither<Error, jwt.JwtPayload>;
export declare const getValidateJWT: GetValidateJWT;
export declare const validateJWTWithKey: (issuer: NonEmptyString, key: NonEmptyString) => (token: NonEmptyString) => TE.TaskEither<Error, jwt.JwtPayload>;
