"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateJWTWithKey = exports.getValidateJWT = exports.errorIsInvalidSignatureError = exports.getGenerateJWT = void 0;
/* eslint-disable @typescript-eslint/no-use-before-define */
const jwt = require("jsonwebtoken");
const ulid_1 = require("ulid");
const function_1 = require("fp-ts/lib/function");
const TE = require("fp-ts/TaskEither");
const E = require("fp-ts/Either");
const O = require("fp-ts/Option");
const strings_1 = require("./strings");
// --------------------
const alg = "RS256";
const getGenerateJWT = (issuer, primaryPrivateKey) => (payload, ttl) => (0, function_1.pipe)(TE.taskify((cb) => jwt.sign(payload, primaryPrivateKey, {
    algorithm: alg,
    expiresIn: `${ttl} seconds`,
    issuer,
    jwtid: (0, ulid_1.ulid)(),
}, cb))(), TE.mapLeft(E.toError), TE.chain(TE.fromPredicate(strings_1.NonEmptyString.is, () => E.toError("Token is empty."))));
exports.getGenerateJWT = getGenerateJWT;
const errorIsInvalidSignatureError = (error) => error.message === "JsonWebTokenError - invalid signature";
exports.errorIsInvalidSignatureError = errorIsInvalidSignatureError;
const getValidateJWT = (issuer, primaryPublicKey, secondaryPublicKey) => (token) => (0, function_1.pipe)(token, (0, exports.validateJWTWithKey)(issuer, primaryPublicKey), TE.orElse((err) => (0, function_1.pipe)(secondaryPublicKey, O.fromNullable, O.chain(O.fromPredicate(() => (0, exports.errorIsInvalidSignatureError)(err))), O.map((key) => (0, exports.validateJWTWithKey)(issuer, key)(token)), O.getOrElse(() => TE.left(err)))));
exports.getValidateJWT = getValidateJWT;
// -----------------------
// Private methods
// -----------------------
const validateJWTWithKey = (issuer, key) => (token) => (0, function_1.pipe)(TE.tryCatch(() => new Promise((resolve, reject) => {
    jwt.verify(token, key, { algorithms: [alg], issuer }, (err, decoded) => {
        if (err) {
            reject(new Error(`${err.name} - ${err.message}`));
        }
        else if (!decoded) {
            reject("Unable to decode token");
        }
        else {
            resolve(decoded);
        }
    });
}), E.toError));
exports.validateJWTWithKey = validateJWTWithKey;
//# sourceMappingURL=jwt_with_key_rotation.js.map