import * as t from "io-ts";
import { Tagged } from "./types";
export interface IWithinRangeNumberTag<L extends number, H extends number> {
    readonly lower: L;
    readonly higher: H;
    readonly kind: "IWithinRangeNumberTag";
}
/**
 * A number guaranteed to be within the range [L,H)
 */
export declare const WithinRangeNumber: <L extends number, H extends number, T extends IWithinRangeNumberTag<L, H>>(l: L, h: H) => Tagged<T, number, number, unknown>;
export type WithinRangeNumber<L extends number, H extends number> = number & IWithinRangeNumberTag<L, H>;
export interface INonNegativeNumberTag {
    readonly kind: "INonNegativeNumberTag";
}
/**
 * A non negative number
 */
export declare const NonNegativeNumber: Tagged<INonNegativeNumberTag, number, number, unknown>;
export type NonNegativeNumber = t.TypeOf<typeof NonNegativeNumber>;
export type Integer = typeof t.Integer;
export interface IWithinRangeIntegerTag<L extends number, H extends number> {
    readonly lower: L;
    readonly higher: H;
    readonly kind: "IWithinRangeIntegerTag";
}
/**
 * An integer guaranteed to be within the range [L,H)
 */
export declare const WithinRangeInteger: <L extends number, H extends number, T extends IWithinRangeIntegerTag<L, H>>(l: L, h: H) => Tagged<T, number, number, unknown>;
export type WithinRangeInteger<L extends number, H extends number> = Integer & IWithinRangeIntegerTag<L, H>;
export interface INonNegativeIntegerTag {
    readonly kind: "INonNegativeIntegerTag";
}
/**
 * A non negative integer
 */
export declare const NonNegativeInteger: Tagged<INonNegativeIntegerTag, number, number, unknown>;
export type NonNegativeInteger = t.TypeOf<typeof NonNegativeInteger>;
/**
 * Parses a string into a decimal
 */
export declare const NumberFromString: t.Type<number, string, unknown>;
export type NumberFromString = t.TypeOf<typeof NumberFromString>;
/**
 * Parses a string into an integer
 */
export declare const IntegerFromString: t.RefinementC<t.Type<number, string, unknown>, number>;
export type IntegerFromString = t.TypeOf<typeof IntegerFromString>;
/**
 * Parses a string into a non negative integer
 */
export declare const NonNegativeIntegerFromString: Tagged<INonNegativeIntegerTag, number, string, unknown>;
