"use strict";
/*
 * Useful tagged types for numbers
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.NonNegativeIntegerFromString = exports.IntegerFromString = exports.NumberFromString = exports.NonNegativeInteger = exports.WithinRangeInteger = exports.NonNegativeNumber = exports.WithinRangeNumber = void 0;
const t = require("io-ts");
const E = require("fp-ts/lib/Either");
const function_1 = require("fp-ts/lib/function");
const types_1 = require("./types");
/**
 * A number guaranteed to be within the range [L,H)
 */
const WithinRangeNumber = (l, h) => (0, types_1.tag)()(t.refinement(t.number, (s) => s >= l && s < h, `number >= ${l} and < ${h}`));
exports.WithinRangeNumber = WithinRangeNumber;
/**
 * A non negative number
 */
exports.NonNegativeNumber = (0, types_1.tag)()(t.refinement(t.number, (s) => s >= 0, "number >= 0"));
/**
 * An integer guaranteed to be within the range [L,H)
 */
const WithinRangeInteger = (l, h) => (0, types_1.tag)()(t.refinement(t.Integer, 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
(s) => s >= l && s < h, `integer >= ${l} and < ${h}`));
exports.WithinRangeInteger = WithinRangeInteger;
/**
 * A non negative integer
 */
exports.NonNegativeInteger = (0, types_1.tag)()(t.refinement(t.Integer, (s) => s >= 0, "integer >= 0"));
/**
 * Parses a string into a decimal
 */
exports.NumberFromString = new t.Type("NumberFromString", t.number.is, (m, c) => (0, function_1.pipe)(t.string.validate(m, c), E.chain((s) => {
    const n = +s;
    return isNaN(n) ? t.failure(s, c) : t.success(n);
})), String);
/**
 * Parses a string into an integer
 */
exports.IntegerFromString = t.refinement(exports.NumberFromString, t.Integer.predicate, "IntegerFromString");
/**
 * Parses a string into a non negative integer
 */
exports.NonNegativeIntegerFromString = (0, types_1.tag)()(t.refinement(exports.IntegerFromString, (i) => i >= 0, "NonNegativeIntegerFromString"));
//# sourceMappingURL=numbers.js.map