"use strict";
/**
 * A type for handling the states of remote (potential) data.
 */
// eslint-disable @typescript-eslint/no-explicit-any
// eslint-disable max-params
// eslint-disable @typescript-eslint/interface-name-prefix
Object.defineProperty(exports, "__esModule", { value: true });
exports.isPot = exports.toOption = exports.toUndefined = exports.orElse = exports.getOrElseWithUpdating = exports.getOrElse = exports.mapNullable = exports.filter = exports.map = exports.fold = exports.toError = exports.toUpdating = exports.toLoading = exports.isError = exports.isUpdating = exports.isLoading = exports.isNone = exports.isSome = exports.toSomeLoading = exports.someError = exports.someUpdating = exports.someLoading = exports.some = exports.noneError = exports.noneUpdating = exports.noneLoading = exports.none = void 0;
const option = require("fp-ts/lib/Option");
exports.none = {
    kind: "PotNone",
};
exports.noneLoading = {
    kind: "PotNoneLoading",
};
const noneUpdating = (newValue) => ({
    kind: "PotNoneUpdating",
    newValue,
});
exports.noneUpdating = noneUpdating;
const noneError = (error) => ({
    error,
    kind: "PotNoneError",
});
exports.noneError = noneError;
const some = (value) => ({
    kind: "PotSome",
    value,
});
exports.some = some;
const someLoading = (value) => ({
    kind: "PotSomeLoading",
    value,
});
exports.someLoading = someLoading;
const someUpdating = (value, newValue) => ({
    kind: "PotSomeUpdating",
    newValue,
    value,
});
exports.someUpdating = someUpdating;
const someError = (value, error) => ({
    error,
    kind: "PotSomeError",
    value,
});
exports.someError = someError;
const toSomeLoading = (// eslint-disable-next-line @typescript-eslint/no-explicit-any
p) => (0, exports.someLoading)(p.value);
exports.toSomeLoading = toSomeLoading;
const isSome = (p) => p.kind === "PotSome" ||
    p.kind === "PotSomeLoading" ||
    p.kind === "PotSomeUpdating" ||
    p.kind === "PotSomeError";
exports.isSome = isSome;
const isNone = (p) => p.kind === "PotNone" ||
    p.kind === "PotNoneLoading" ||
    p.kind === "PotNoneUpdating" ||
    p.kind === "PotNoneError";
exports.isNone = isNone;
const isLoading = (// eslint-disable-next-line @typescript-eslint/no-explicit-any
p) => p.kind === "PotNoneLoading" || p.kind === "PotSomeLoading";
exports.isLoading = isLoading;
const isUpdating = (
// eslint-disable-next-line @typescript-eslint/no-explicit-any
p // eslint-disable-next-line @typescript-eslint/no-explicit-any
) => p.kind === "PotNoneUpdating" || p.kind === "PotSomeUpdating";
exports.isUpdating = isUpdating;
const isError = (p) => p.kind === "PotNoneError" || p.kind === "PotSomeError";
exports.isError = isError;
const toLoading = (
// eslint-disable-next-line @typescript-eslint/no-explicit-any
p) => // eslint-disable-next-line @typescript-eslint/no-explicit-any
 (0, exports.isSome)(p) ? (0, exports.someLoading)(p.value) : exports.noneLoading;
exports.toLoading = toLoading;
const toUpdating = (// eslint-disable-next-line @typescript-eslint/no-explicit-any
p, newValue) => (0, exports.isSome)(p) ? (0, exports.someUpdating)(p.value, newValue) : (0, exports.noneUpdating)(newValue);
exports.toUpdating = toUpdating;
const toError = (p, error) => (0, exports.isSome)(p) ? (0, exports.someError)(p.value, error) : (0, exports.noneError)(error);
exports.toError = toError;
const fold = (p, foldNone, foldNoneLoading, foldNoneUpdating, foldNoneError, foldSome, foldSomeLoading, foldSomeUpdating, foldSomeError
// eslint-disable-next-line max-params
) => {
    // eslint-disable-next-line default-case
    switch (p.kind) {
        case "PotNone":
            return foldNone();
        case "PotNoneLoading":
            return foldNoneLoading();
        case "PotNoneUpdating":
            return foldNoneUpdating(p.newValue);
        case "PotNoneError":
            return foldNoneError(p.error);
        case "PotSome":
            return foldSome(p.value);
        case "PotSomeLoading":
            return foldSomeLoading(p.value);
        case "PotSomeUpdating":
            return foldSomeUpdating(p.value, p.newValue);
        case "PotSomeError":
            return foldSomeError(p.value, p.error);
    }
};
exports.fold = fold;
const map = (p, f) => (0, exports.fold)(p, () => exports.none, () => exports.noneLoading, (newValue) => (0, exports.noneUpdating)(f(newValue)), (error) => (0, exports.noneError)(error), (value) => (0, exports.some)(f(value)), (value) => (0, exports.someLoading)(f(value)), (value, newValue) => (0, exports.someUpdating)(f(value), f(newValue)), (value, error) => (0, exports.someError)(f(value), error));
exports.map = map;
const filter = (p, f) => (0, exports.fold)(p, () => p, () => p, () => p, () => p, (value) => (f(value) ? p : exports.none), (value) => (f(value) ? p : exports.noneLoading), (value, newValue) => (f(value) ? p : (0, exports.noneUpdating)(newValue)), (value, error) => (f(value) ? p : (0, exports.noneError)(error)));
exports.filter = filter;
const mapNullable = (p, f) => {
    const mapped = (0, exports.map)(p, f);
    return (0, exports.filter)(mapped, (_) => _ !== undefined && _ !== null);
};
exports.mapNullable = mapNullable;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const getOrElse = (p, o) => (0, exports.isSome)(p) ? p.value : o;
exports.getOrElse = getOrElse;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const getOrElseWithUpdating = (p, o) => (0, exports.isUpdating)(p) ? p.newValue : (0, exports.isSome)(p) ? p.value : o;
exports.getOrElseWithUpdating = getOrElseWithUpdating;
const orElse = (p, o) => (0, exports.isSome)(p) ? p : o;
exports.orElse = orElse;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const toUndefined = (p) => (0, exports.isSome)(p) ? p.value : undefined;
exports.toUndefined = toUndefined;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const toOption = (p) => option.fromNullable((0, exports.toUndefined)(p));
exports.toOption = toOption;
const PotKinds = {
    PotNone: 0,
    PotNoneError: 0,
    PotNoneLoading: 0,
    PotNoneUpdating: 0,
    PotSome: 0,
    PotSomeError: 0,
    PotSomeLoading: 0,
    PotSomeUpdating: 0,
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isPot = (value) => value !== null &&
    typeof value === "object" &&
    value.kind !== undefined &&
    value.kind in PotKinds;
exports.isPot = isPot;
//# sourceMappingURL=pot.js.map