import { Either } from "fp-ts/lib/Either";
import { ITuple3 } from "./tuples";
import { Millisecond } from "./units";
/**
 * Useful Promises combinators
 */
/**
 * Returns a Promise that resolves after millis milliseconds
 */
export declare const timeoutPromise: (millis: Millisecond) => Promise<void>;
/**
 * Wraps a Promise with a timeout, resolves to Left<"timeout"> on timeout or
 * Right<T> on completion.
 */
export declare const withTimeout: <T>(p: Promise<T>, millis: Millisecond, timeoutP?: (_: Millisecond) => Promise<void>) => Promise<Either<"timeout", T>>;
/**
 * Creates a Promise whose resolve and reject implementations are not yet
 * provided.
 *
 * @return Array  A triplet with [Promise<T>, resolve(T), reject()]
 */
export declare const DeferredPromise: <T>() => ITuple3<Promise<T>, (v: T) => void, (e: Error) => void>;
