"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeferredPromise = exports.withTimeout = exports.timeoutPromise = void 0;
const Either_1 = require("fp-ts/lib/Either");
const tuples_1 = require("./tuples");
/**
 * Useful Promises combinators
 */
/**
 * Returns a Promise that resolves after millis milliseconds
 */
const timeoutPromise = (millis) => new Promise((resolve) => {
    setTimeout(() => resolve(), millis);
});
exports.timeoutPromise = timeoutPromise;
/**
 * Wraps a Promise with a timeout, resolves to Left<"timeout"> on timeout or
 * Right<T> on completion.
 */
const withTimeout = (p, millis, timeoutP = exports.timeoutPromise) => {
    const t = timeoutP(millis);
    return new Promise((resolve, reject) => {
        // on timeout
        t.then(() => resolve((0, Either_1.left)("timeout"))).catch(reject);
        // on completion
        p.then((v) => resolve((0, Either_1.right)(v))).catch(reject);
    });
};
exports.withTimeout = withTimeout;
/**
 * Creates a Promise whose resolve and reject implementations are not yet
 * provided.
 *
 * @return Array  A triplet with [Promise<T>, resolve(T), reject()]
 */
const DeferredPromise = () => {
    // eslint-disable-next-line functional/no-let
    let resolvePromise = () => {
        throw new Error("Promise.resolve not yet initialized");
    };
    // eslint-disable-next-line functional/no-let
    let rejectPromise = () => {
        throw new Error("Promise.reject not yet initialized");
    };
    // eslint-disable-next-line @typescript-eslint/no-floating-promises
    const promise = new Promise((resolve, reject) => {
        resolvePromise = resolve;
        rejectPromise = reject;
    });
    return (0, tuples_1.Tuple3)(promise, resolvePromise, rejectPromise);
};
exports.DeferredPromise = DeferredPromise;
//# sourceMappingURL=promises.js.map