"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReadableReporterSimplified = exports.readableReportSimplified = exports.ReadableReporter = exports.readableReport = exports.errorsToReadableMessages = void 0;
const E = require("fp-ts/lib/Either");
const function_1 = require("fp-ts/lib/function");
const string_1 = require("fp-ts/lib/string");
/**
 * Translate a context to a more readable path.
 *
 * e.g.:
 *
 *   "is not a non empty string"
 *   ".a is not a number"
 *   ".c.b is not a non empty string"
 */
const getContextPath = (context) => {
    if (context.length === 0) {
        return "] (decoder info n/a)";
    }
    const keysPath = context.map(({ key }) => key).join(".");
    const lastType = context[context.length - 1].type;
    if ("never" === lastType.name) {
        return `${keysPath}] is not a known property`;
    }
    return `${keysPath}] is not a valid [${lastType.name}]`;
};
/**
format path without usless indexes
 */
const isArrayIndex = (prev, c, i, context) => c.key !== "" &&
    Number.isInteger(+c.key) &&
    i > 0 &&
    (0, string_1.includes)("Array")(context[i - 1].type.name);
const getContextPathSimplified = (context) => {
    if (context.length === 0) {
        return " (decoder info n/a)";
    }
    /*
      in order to get a more readable ouptut we want to avoid those keys that represent the index of a t.union or a t.intersection,
      to do this we want to render the key in the output just in 2 cases:
        1. the key can be parsed as a valid integer and the element right before was an ArrayType
        2. the key is not parsable as a valid integer but it is also a non empty string
    */
    const keysPath = context.reduce((prev, c, i) => isArrayIndex(prev, c, i, context) || !Number.isInteger(+c.key)
        ? Number.isInteger(+c.key)
            ? // key is an integer and the previous element is an array, we keep it
                [...prev, `[${c.key}]`]
            : // key is not an integer, we keep it (dotted notation)
                [...prev, `.${c.key}`]
        : // key is an integer but the previous element is not an array, we skip it
            prev, []);
    const lastType = context[context.length - 1].type;
    if ("never" === lastType.name) {
        return `${keysPath.join("")} is not a known property`;
    }
    return `${keysPath.join("")} is not a valid [${lastType.name}]`;
};
const getMessage = (value, context) => `value [${JSON.stringify(value)}] at [root${getContextPath(context)}`;
const getMessageSimplified = (value, context) => `value ${JSON.stringify(value)} at root${getContextPathSimplified(context)}`;
/**
 * Translates validation errors to more readable messages.
 */
const errorsToReadableMessages = (es, isSimplified = false) => isSimplified
    ? es.map((e) => getMessageSimplified(e.value, e.context))
    : es.map((e) => getMessage(e.value, e.context));
exports.errorsToReadableMessages = errorsToReadableMessages;
const success = () => ["No errors!"];
const readableReport = (errors) => (0, exports.errorsToReadableMessages)(errors).join("\n");
exports.readableReport = readableReport;
/**
 * A validation error reporter that translates validation errors to more
 * readable messages.
 */
exports.ReadableReporter = {
    report: (validation) => (0, function_1.pipe)(validation, E.fold(exports.errorsToReadableMessages, success)),
};
const readableReportSimplified = (errors) => (0, exports.errorsToReadableMessages)(errors, true).join("\n");
exports.readableReportSimplified = readableReportSimplified;
/**
 * A validation error reporter that translates validation errors to more
 * readable messages avoiding indexes for types such as t.intersection or t.union.
 */
exports.ReadableReporterSimplified = {
    report: (validation) => (0, function_1.pipe)(validation, E.fold((v) => (0, exports.errorsToReadableMessages)(v, true), success)),
};
//# sourceMappingURL=reporters.js.map