"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withRequestMiddlewares = exports.wrapRequestHandler = void 0;
const E = require("fp-ts/lib/Either");
const TE = require("fp-ts/lib/TaskEither");
const function_1 = require("fp-ts/lib/function");
const responses_1 = require("./responses");
/**
 * Transforms a typesafe RequestHandler into an Express Request Handler.
 *
 * Failed promises will be mapped to 500 errors handled by ResponseErrorGeneric.
 */
const wrapRequestHandler = (handler) => (request, response, _) => handler(request).then((r) => {
    r.apply(response);
}, (e) => {
    (0, responses_1.ResponseErrorInternal)(e).apply(response);
});
exports.wrapRequestHandler = wrapRequestHandler;
const withRequestMiddlewares = (...middlewares) => (handler) => 
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
(request) => (0, function_1.pipe)(middlewares.map((middleware) => (0, function_1.pipe)(TE.tryCatch(() => middleware(request), (_) => (0, responses_1.ResponseErrorInternal)(`error executing middleware`)), TE.chain(TE.fromEither))), TE.sequenceSeqArray, TE.chain((params) => (0, function_1.pipe)(TE.tryCatch(() => handler(...params), E.toError), TE.mapLeft((err) => (0, responses_1.ResponseErrorInternal)(`Error executing endpoint handler: ${err.message}`)))), TE.toUnion)();
exports.withRequestMiddlewares = withRequestMiddlewares;
//# sourceMappingURL=request_middleware.js.map