"use strict";
/**
 * Type safe wrapper around the fetch API
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthorizationBearerHeaderProducer = exports.ParamAuthorizationBearerHeaderProducer = exports.constantResponseDecoder = exports.basicResponseDecoder = exports.basicErrorResponseDecoder = exports.bufferArrayResponseDecoder = exports.ioResponseDecoder = exports.ApiHeaderJson = exports.createFetchRequestForApi = exports.ApiRequestBuilder = exports.composeResponseDecoders = exports.composeHeaderProducers = void 0;
// eslint-disable camelcase
// TODO: support for optional query parameters
// TODO: when query/headers type is "never", it should not allow any query/header to be produced
// TODO: add etag support in responses
const Either_1 = require("fp-ts/lib/Either");
const E = require("fp-ts/lib/Either");
const function_1 = require("fp-ts/lib/function");
/**
 * Composes two RequestHeaderProducer(s)
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function composeHeaderProducers(p0, p1) {
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (params) => {
        const headers0 = p0(params);
        const headers1 = p1(params);
        return Object.assign(Object.assign({}, headers0), headers1);
    };
}
exports.composeHeaderProducers = composeHeaderProducers;
/**
 * Composes two ResponseDecoder(s)
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function composeResponseDecoders(d1, d2) {
    // TODO: make sure R1, R2 don't intersect
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (response) => __awaiter(this, void 0, void 0, function* () {
        const r1 = yield d1(response);
        return r1 !== undefined ? r1 : yield d2(response);
    });
}
exports.composeResponseDecoders = composeResponseDecoders;
const emptyBaseRequest = {
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    headers: () => ({}),
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    query: () => ({}),
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    response_decoder: () => Promise.reject({}),
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    url: () => "",
};
const emptyBasePutOrPostRequest = Object.assign(Object.assign({}, emptyBaseRequest), { 
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    body: () => JSON.stringify({}), 
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type, @typescript-eslint/naming-convention
    headers: () => ({ "Content-Type": "application/json" }) });
/**
 * A class for building ApiRequest(s)
 */
class ApiRequestBuilder {
    constructor(_request) {
        this._request = _request;
    }
    /**
     * Creates a new empty request with the GET method.
     */
    static ofGet() {
        return new ApiRequestBuilder(Object.assign(Object.assign({}, emptyBaseRequest), { method: "get" }));
    }
    /**
     * Creates a new empty request with the POST method.
     */
    // eslint-disable-next-line @typescript-eslint/ban-types
    static ofPost() {
        return new ApiRequestBuilder(Object.assign(Object.assign({}, emptyBasePutOrPostRequest), { method: "post" }));
    }
    /**
     * Creates a new empty request with the PUT method.
     */
    /* eslint-disable-next-line sonarjs/no-identical-functions */
    // eslint-disable-next-line @typescript-eslint/ban-types
    static ofPut() {
        return new ApiRequestBuilder(Object.assign(Object.assign({}, emptyBasePutOrPostRequest), { method: "put" }));
    }
    get() {
        // eslint-disable-next-line no-underscore-dangle
        return this._request;
    }
    /**
     * Adds query parameters
     */
    withQuery(query) {
        // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
        const newQuery = (p) => (Object.assign(Object.assign({}, this._request.query(p)), query(p)));
        return new ApiRequestBuilder(Object.assign(Object.assign({}, this._request), { query: newQuery }));
    }
    /**
     * Adds headers
     */
    withHeaders(headers) {
        // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
        const newHeaders = (p) => (Object.assign(Object.assign({}, this._request.headers(p)), headers(p)));
        return new ApiRequestBuilder(Object.assign(Object.assign({}, this._request), { headers: newHeaders }));
    }
    /**
     * Adds response decoder
     */
    withResponseDecoder(response_decoder) {
        return new ApiRequestBuilder(Object.assign(Object.assign({}, this._request), { response_decoder }));
    }
}
exports.ApiRequestBuilder = ApiRequestBuilder;
//
// helpers
//
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function queryStringFromParams(params) {
    const keys = Object.getOwnPropertyNames(params);
    if (keys.length === 0) {
        return undefined;
    }
    return keys
        .map((k) => `${encodeURIComponent(k)}=${encodeURIComponent(params[k])}`)
        .join("&");
}
/**
 * Returns an async method that implements the provided ApiRequestType backed
 * by the "fetch" API.
 */
// eslint-disable-next-line sonarjs/cognitive-complexity, prefer-arrow/prefer-arrow-functions
function createFetchRequestForApi(requestType, options = {}) {
    // TODO: handle unsuccessful fetch and HTTP errors
    // @see https://www.pivotaltracker.com/story/show/154661120
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (params) => __awaiter(this, void 0, void 0, function* () {
        // Build operationUrl from the params
        const operationUrl = requestType.url(params);
        // Build request url from baseUrl if provided
        const requestUrl = options.baseUrl
            ? options.baseUrl +
                (options.baseUrl.endsWith("/") || operationUrl.startsWith("/")
                    ? ""
                    : "/") +
                operationUrl
            : operationUrl;
        // Generate the query params
        const queryParams = requestType.query(params);
        const queryString = queryStringFromParams(queryParams);
        // Append the query params to the URL
        const url = queryString === undefined ? requestUrl : `${requestUrl}?${queryString}`;
        // get the headers from the params
        const headers = requestType.headers(params);
        // build the request
        const baseRequest = {
            method: requestType.method,
        };
        const requestWithOptionalHeaders = Object.keys(headers).length === 0
            ? baseRequest
            : Object.assign(Object.assign({}, baseRequest), { headers });
        const requestWithOptionalHeadersAndBody = requestType.method === "get" || requestType.method === "delete"
            ? requestWithOptionalHeaders
            : Object.assign(Object.assign({}, requestWithOptionalHeaders), { body: requestType.body(params) });
        // Get the fetch client
        const myFetch = options.fetchApi ? options.fetchApi : fetch;
        // make the async call
        const response = yield myFetch(url, requestWithOptionalHeadersAndBody);
        // decode the response
        const decoded = yield requestType.response_decoder(response);
        return decoded !== undefined
            ? decoded
            : (0, Either_1.left)([
                {
                    context: [],
                    value: response,
                },
            ]);
    });
}
exports.createFetchRequestForApi = createFetchRequestForApi;
/**
 * An header producer that sets the Content-Type to application/json
 */
// eslint-disable-next-line @typescript-eslint/ban-types
const ApiHeaderJson = () => ({
    // eslint-disable-next-line @typescript-eslint/naming-convention
    "Content-Type": "application/json",
});
exports.ApiHeaderJson = ApiHeaderJson;
/**
 * An io-ts based ResponseDecoder, it checks the response status and the
 * payload against the provided type.
 *
 * @param status  The response status handled by this decoder
 * @param type    The response type corresponding to the status
 * @param preprocessor A function that takes the object corresponding to the json response as input
 * and returns the processed object (usefull when you want alterate the json body received)
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function ioResponseDecoder(status, type, 
// eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/explicit-function-return-type
preprocessor = (_) => _) {
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (response) => __awaiter(this, void 0, void 0, function* () {
        if (response.status !== status) {
            // skip this decoder if status doesn't match
            return undefined;
        }
        const json = yield response.json();
        const validated = type.decode(preprocessor(json));
        return (0, function_1.pipe)(validated, E.map((value) => ({
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            headers: response.headers,
            status,
            value,
        })));
    });
}
exports.ioResponseDecoder = ioResponseDecoder;
/**
 * An arrayBuffer ResponseDecoder
 * it returns an IResponse with a Buffer value
 *
 * @param status  The response status handled by this decoder
 */
const bufferArrayResponseDecoder = (status) => (response) => __awaiter(void 0, void 0, void 0, function* () {
    if (response.status !== status) {
        // skip this decoder if status doesn't match
        return undefined;
    }
    const arrayBuffer = yield response.arrayBuffer();
    return (0, function_1.pipe)(arrayBuffer, E.of, E.map((ab) => Buffer.from(ab)), E.map((buffer) => ({
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        headers: response.headers,
        status,
        value: buffer,
    })));
});
exports.bufferArrayResponseDecoder = bufferArrayResponseDecoder;
/**
 * A basic ResponseDecoder that returns an Error with the status text if the
 * response status is S.
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function basicErrorResponseDecoder(status) {
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (response) => __awaiter(this, void 0, void 0, function* () {
        if (response.status !== status) {
            // skip this decoder if status doesn't match
            return undefined;
        }
        return (0, Either_1.right)({
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            headers: response.headers,
            status,
            value: response.statusText,
        });
    });
}
exports.basicErrorResponseDecoder = basicErrorResponseDecoder;
/**
 * Returns a ResponseDecoder for BasicResponseType<R>
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function basicResponseDecoder(type, 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
preprocessor) {
    return composeResponseDecoders(composeResponseDecoders(ioResponseDecoder(200, type, preprocessor), basicErrorResponseDecoder(404)), basicErrorResponseDecoder(500));
}
exports.basicResponseDecoder = basicResponseDecoder;
/**
 * A response decoder that ignores the payload and returns a constant value
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function constantResponseDecoder(status, value) {
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (response) => __awaiter(this, void 0, void 0, function* () {
        if (response.status !== status) {
            return undefined;
        }
        return (0, Either_1.right)({
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            headers: response.headers,
            status,
            value,
        });
    });
}
exports.constantResponseDecoder = constantResponseDecoder;
/**
 * Returns a RequestHeaderProducer that produces an Authorization header of type
 * "bearer token" taking the value from the "token" parameter of each request.
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function ParamAuthorizationBearerHeaderProducer() {
    return (p) => ({
        Authorization: `Bearer ${p.token}`,
    });
}
exports.ParamAuthorizationBearerHeaderProducer = ParamAuthorizationBearerHeaderProducer;
/**
 * Returns a RequestHeaderProducer that produces an Authorization header of type
 * "bearer token" with a fixed token value.
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function AuthorizationBearerHeaderProducer(token) {
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return () => ({
        Authorization: `Bearer ${token}`,
    });
}
exports.AuthorizationBearerHeaderProducer = AuthorizationBearerHeaderProducer;
//# sourceMappingURL=requests.js.map