import { Task } from "fp-ts/lib/Task";
import { TaskEither } from "fp-ts/lib/TaskEither";
import { Millisecond } from "./units";
/**
 * Returns a Task that resolves to a value after a delay.
 */
export declare const delayTask: <A>(n: Millisecond, a: A) => Task<A>;
/**
 * In the context of a retriable task, when it returns a TransientError the
 * task can be executed again.
 */
export type TransientError = "transient";
export declare const TransientError: TransientError;
/**
 * A RetriableTask failed too many times.
 */
export type MaxRetries = "max-retries";
export declare const MaxRetries: MaxRetries;
/**
 * A RetriableTask has been aborted.
 */
export type RetryAborted = "retry-aborted";
export declare const RetryAborted: RetryAborted;
/**
 * A Task that can be retried when it fails with a transient error
 */
export type RetriableTask<E, T> = E extends TransientError ? never : TaskEither<E | TransientError, T>;
/**
 * Wraps a RetriableTask with a number of retries
 */
export declare const withRetries: <E, T>(maxRetries: number, backoff: (count: number) => Millisecond) => (_: RetriableTask<E, T>, shouldAbort?: Promise<boolean>) => TaskEither<"max-retries" | "retry-aborted" | E, T>;
