"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withRetries = exports.RetryAborted = exports.MaxRetries = exports.TransientError = exports.delayTask = void 0;
const Either_1 = require("fp-ts/lib/Either");
const T = require("fp-ts/lib/Task");
const TE = require("fp-ts/lib/TaskEither");
const function_1 = require("fp-ts/lib/function");
/**
 * Returns a Task that resolves to a value after a delay.
 */
const delayTask = (n, a) => T.delay(n)(T.of(a));
exports.delayTask = delayTask;
exports.TransientError = "transient";
exports.MaxRetries = "max-retries";
exports.RetryAborted = "retry-aborted";
/**
 * Wraps a RetriableTask with a number of retries
 */
const withRetries = (maxRetries, backoff) => 
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
(task, shouldAbort = Promise.resolve(false)) => {
    // Whether we must stop retrying
    // the abort process gets triggered when the shouldAbort promise resolves
    // to true. Not that aborting stops the retry process, it does NOT stop
    // the execution of the current task.
    // eslint-disable-next-line functional/no-let
    let mustAbort = false;
    shouldAbort.then((v) => {
        mustAbort = v;
    }, (_) => void 0);
    const runTaskOnce = (count, currentTask) => {
        // on first execution, count === 0
        if (count >= maxRetries - 1) {
            // no more retries left
            return currentTask;
        }
        // allow one run of the task
        return (0, function_1.pipe)(currentTask, TE.orElse((l) => {
            // if the task fails...
            if (mustAbort) {
                return TE.left(exports.RetryAborted);
            }
            if (l === exports.TransientError) {
                // ...with a TransientError, chain it with a backoff delay
                // an then with another run.
                return (0, function_1.pipe)((0, exports.delayTask)(backoff(count), true), T.map((d) => (0, Either_1.right)(d)), TE.chain(() => runTaskOnce(count + 1, currentTask)));
            }
            // ...with an error that is not a TransientError, we just return it
            return TE.left(l);
        }));
    };
    // if the recursive task execution returns with a failure and the failure is
    // a TransientError, it means that the retries have been exausted - we map
    // the error to a MaxRetries error.
    return (0, function_1.pipe)(runTaskOnce(0, task), TE.mapLeft((l) => (l === exports.TransientError ? exports.MaxRetries : l)));
};
exports.withRetries = withRetries;
//# sourceMappingURL=tasks.js.map