"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.replaceProp1 = exports.requiredProp1 = exports.withDefault = exports.strictInterfaceWithOptionals = exports.withoutUndefinedValues = exports.isObject = exports.readonlyNonEmptySetType = exports.readonlySetType = exports.enumType = exports.untag = exports.tag = exports.pick = void 0;
const t = require("io-ts");
const E = require("fp-ts/lib/Either");
const json_set_map_1 = require("json-set-map");
const function_1 = require("fp-ts/lib/function");
/**
 * Returns a subset of the input objects fields.
 *
 * @param obj the input object
 * @param props an array of keys to preserve
 */
const pick = (props, obj) => props.reduce((result, key) => Object.assign({}, result, { [key]: obj[key] }), {});
exports.pick = pick;
/**
 * Tags an io-ts type with an interface T
 */
const tag = () => (type) => 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
type;
exports.tag = tag;
/**
 * Returns the passed tagged basic value after converting it to its basic type.
 */
const untag = (a) => 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
a;
exports.untag = untag;
/**
 * Returns an object where the keys are the values
 * of the object passed as input and all values are undefined
 */
// eslint-disable-next-line @typescript-eslint/ban-types
const getObjectValues = (e) => Object.keys(e).reduce(
// eslint-disable-next-line @typescript-eslint/no-explicit-any
(o, k) => (Object.assign(Object.assign({}, o), { [e[k]]: undefined })), {});
/**
 * Creates an io-ts Type from an enum
 */
// eslint-disable-next-line @typescript-eslint/ban-types
const enumType = (e, name) => 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
t.keyof(getObjectValues(e), name);
exports.enumType = enumType;
/**
 * Creates an io-ts Type from a ReadonlySet
 */
const readonlySetType = (o, name) => {
    const arrayType = t.readonlyArray(o, name);
    return new t.Type(name, (s) => s instanceof Set && arrayType.is(Array.from(s)), (s, c) => {
        if ((s instanceof Set && arrayType.is(Array.from(s))) ||
            arrayType.is(s)) {
            return t.success(new json_set_map_1.Set(Array.from(s)));
        }
        return t.failure(s, c);
    }, t.identity);
};
exports.readonlySetType = readonlySetType;
/**
 * Creates an io-ts Type from a ReadonlyNonEmptySet
 */
const readonlyNonEmptySetType = (o, name) => t.refinement((0, exports.readonlySetType)(o, name), (e) => e.size > 0, name);
exports.readonlyNonEmptySetType = readonlyNonEmptySetType;
/**
 *  True when the input is an object (and not array).
 */
// eslint-disable-next-line @typescript-eslint/ban-types
const isObject = (o) => o instanceof Object && o.constructor === Object;
exports.isObject = isObject;
/**
 * Return an object filtering out keys that point to undefined values.
 */
// eslint-disable-next-line
const withoutUndefinedValues = (obj) => {
    // note that T has been already validated by the type system and we can
    // be sure now that only attributes that may be undefined can be actually
    // filtered out by the following code, so the output type T is always
    // a valid T
    const keys = Object.keys(obj);
    return keys.reduce((acc, key) => {
        const value = obj[key];
        return value !== undefined && value !== null
            ? Object.assign(Object.assign({}, acc), { 
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                [key]: (0, exports.isObject)(value) ? (0, exports.withoutUndefinedValues)(value) : value }) : acc;
    }, {});
};
exports.withoutUndefinedValues = withoutUndefinedValues;
/**
 *  Return a new type that validates successfully only
 *  when the instance (object) contains no unknown properties.
 *
 *  See https://github.com/gcanti/io-ts/issues/106
 *
 *  @\required  required properties
 *  @optional   optional object properties
 */
const strictInterfaceWithOptionals = (required, optional, name) => {
    const loose = t.intersection([t.interface(required), t.partial(optional)]);
    const props = Object.assign({}, required, optional);
    return new t.Type(name, (m) => loose.is(m) &&
        // check if all object properties belong to the strict interface
        // eslint-disable-next-line no-prototype-builtins
        Object.getOwnPropertyNames(m).every((k) => props.hasOwnProperty(k)), (m, c) => (0, function_1.pipe)(loose.validate(m, c), E.chain((o) => {
        const errors = Object.getOwnPropertyNames(o)
            .map((key) => 
        // eslint-disable-next-line no-prototype-builtins
        !props.hasOwnProperty(key)
            ? t.getValidationError(o[key], t.appendContext(c, key, t.never))
            : undefined)
            .filter((e) => e !== undefined);
        return errors.length ? t.failures(errors) : t.success(o);
    })), loose.encode);
};
exports.strictInterfaceWithOptionals = strictInterfaceWithOptionals;
/**
 * Sets properties default values when calling t.validate() method on models
 * see https://github.com/gcanti/io-ts/issues/8
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const withDefault = (type, defaultValue
// eslint-disable-next-line @typescript-eslint/no-explicit-any
) => new t.Type(type.name, 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
(v) => type.is(v), 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
(v, c) => type.validate(v !== undefined && v !== null ? v : defaultValue, c), 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
(v) => type.encode(v));
exports.withDefault = withDefault;
const requiredProp1 = (type, p, name) => 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
t.refinement(type, (o) => o[p] !== undefined, name);
exports.requiredProp1 = requiredProp1;
const replaceProp1 = (type, p, typeB, name) => 
// eslint-disable-next-line @typescript-eslint/no-explicit-any
t.refinement(type, (o) => typeB.is(o[p]), name);
exports.replaceProp1 = replaceProp1;
//# sourceMappingURL=types.js.map